/*
 * Copyright (c) 2016-2017 Satlab Aps <satlab@satlab.com>
 * All rights reserved. Do not distribute without permission.
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <inttypes.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <csp/csp_endian.h>

#include <slash/slash.h>

#include <satlab/srs3.h>
#include <prop-client/prop_client_commands.h>

/* Default node and timeout */
static uint8_t srs3_node = SL_SRS3_DEFAULT_ADDRESS;
static uint32_t srs3_timeout = 1000;

slash_command_group(srs3, "SRS-3 subsystem commands");

static int cmd_srs3_node(struct slash *slash)
{
	uint8_t node;
	char *endptr;

	if (slash->argc < 2) {
		printf("Current node address: %hhu\n", srs3_node);
		return 0;
	}

	node = (uint8_t)strtoul(slash->argv[1], &endptr, 10);
	if (*endptr != '\0')
		return SLASH_EUSAGE;

	srs3_node = node;

	return 0;
}
slash_command_sub(srs3, node, cmd_srs3_node, "[address]",
		  "Set/read node address");

static int cmd_srs3_timeout(struct slash *slash)
{
	uint32_t timeout;
	char *endptr;

	if (slash->argc < 2) {
		printf("Current timeout: %"PRIu32"\n", srs3_timeout);
		return 0;
	}

	timeout = (uint32_t)strtoul(slash->argv[1], &endptr, 10);
	if (*endptr != '\0')
		return SLASH_EUSAGE;

	srs3_timeout = timeout;

	return 0;
}
slash_command_sub(srs3, timeout, cmd_srs3_timeout, "[ms]",
		  "Set/read communication timeout");

slash_command_subgroup(srs3, gwdt, "Ground watchdog commands");

static int cmd_srs3_gwdt_reset(struct slash *slash)
{
	int ret;

	ret = sl_srs3_gwdt_reset(srs3_node, srs3_timeout, SL_SRS3_GWDT_RESET_KEY);
	if (ret < 0) {
		printf("Failed to reset GWDT: %d\n", ret);
		return SLASH_EIO;
	}

	return 0;
}
slash_command_subsub(srs3, gwdt, reset, cmd_srs3_gwdt_reset, NULL,
		     "Reset ground watchdog");

static int cmd_srs3_gwdt_counter(struct slash *slash)
{
	int ret;
	uint32_t counter;

	ret = sl_srs3_gwdt_counter(srs3_node, srs3_timeout, &counter);
	if (ret < 0) {
		printf("Failed to read GWDT: %d\n", ret);
		return SLASH_EIO;
	}

	printf("%"PRIu32" seconds remaining\n", counter);

	return 0;
}
slash_command_subsub(srs3, gwdt, counter, cmd_srs3_gwdt_counter, NULL,
		     "Read ground watchdog counter");

static int cmd_srs3_run(struct slash *slash)
{
	int ret;
	const char *cmd;

	if (slash->argc != 2)
		return SLASH_EUSAGE;

	cmd = slash->argv[1];

	ret = sl_srs3_shell_run(srs3_node, srs3_timeout, cmd);
	if (ret < 0) {
		printf("Failed to run command: %s\n", strerror(-ret));
		return SLASH_EIO;
	}

	return 0;
}
slash_command_sub(srs3, run, cmd_srs3_run, "<command>",
		  "Run command on remote system");

/* Properties commands */
SL_PROP_CLIENT_COMMANDS_GENERATE(srs3, &srs3_node, &srs3_timeout, &srs3_props);
